clear all; clc

% This code is part of the work: "Impedance spectroscopy analysis of thermoelectric modules under actual energy harvesting operating
% conditions and a small temperature difference" by Braulio Beltrn-Pitarch, Francisco Vidan, Marc Carb and Jorge Garca-Caadas

% The code performs a fitting to the experimental data (provided by the user) of a sandwiched TE module using the equivalent circuit of
% Fig. 3 or Fig. 7, depending on the operaing mode.

% To introduce the data follow the next steps:

% 1.- Create a .txt file with name Example.txt introducing the data of the TE module to evaluate. It has to contain 3 tab-delimited columns,
% the first column must be the frequency, the second must be the real part of the impedance, and the third must be the imaginary part. 
% As an example:

% 1000000	2.6727	2.0503
% 686650	2.3348	1.4682
% 471490	2.1673	1.03
% 323750	2.0844	0.71864
% 222300	2.0411	0.50019
% 152640	2.0267	0.34747
% 104810	2.014	0.24009
% 71969	2.0088	0.1621
% 49417	2.0061	0.11015
% 33932	2.0059	0.074041
% 23300	2.0068	0.049143
% 15999	2.0084	0.032076
% 10985	2.0101	0.020498
% 7543.1	2.0098	0.012562
% 5179.5	2.0111	0.0071148
% 3556.5	2.012	0.0031936
% 2442.1	2.0128	0.00030262
% 1676.8	2.0135	-0.0018978
% 1151.4	2.0144	-0.00367
% 790.6	2.0156	-0.0049584
% 542.87	2.0173	-0.0060208
% 372.76	2.0193	-0.0067979
% 255.95	2.0217	-0.0073832
% 175.75	2.0244	-0.0079354
% 120.68	2.0273	-0.0085252
% 82.864	2.0298	-0.0091093
% 56.899	2.032	-0.0094687
% 39.069	2.0339	-0.010035
% 26.827	2.036	-0.010634
% 18.421	2.0385	-0.011879
% 12.649	2.0413	-0.013157
% 8.6851	2.0437	-0.014701
% 5.9636	2.0466	-0.016474
% 4.0949	2.0487	-0.019178
% 2.8118	2.0504	-0.023687
% 1.9307	2.0521	-0.030576
% 1.3257	2.053	-0.041131
% 0.9103	2.0562	-0.058153
% 0.62506	2.0583	-0.079167
% 0.42919	2.0619	-0.117
% 0.29471	2.0758	-0.17252
% 0.20236	2.096	-0.24521
% 0.13895	2.1507	-0.34264
% 0.09541	2.2208	-0.44386
% 0.065513	2.3892	-0.58876
% 0.044984	2.5605	-0.70337
% 0.030888	2.8073	-0.71554
% 0.02121	3.0799	-0.6587
% 0.014563	3.2924	-0.55202
% 0.01	3.4232	-0.42626

% 2.- Save this file (Manuscript_Fitting_DT_v1.m) and the created .txt file (Example.txt) in the same directory.

% 3.- Run this code.

name_file='Qinconstant_0A.txt';

% ---------------------------------------------------------- NO NEED TO MODIFY ANYTHING ELSE ----------------------------------------------------------
% -----------------------------------------------------------------------------------------------------------------------------------------------------

struct1=load(name_file);
dd= {'L (H)','Rohm (Ohm)','rTC1 (m^2K/W)','rTC2 (m^2K/W)','rTC (m^2K/W)','hSO (W/m^2K)','S (V/K)','lambdaTE (W/(mK))','alphaTE (m^2/s)','lambdaM (W/(mK))',...
    'alphaM (m^2/s)','lambdaC (W/(mK))','alphaC (m^2/s)','lambdaSo (W/(mK))','alphaSo (m^2/s)','Err_L (%)','Err_Rohm (%)','Err_rTC1 (%)','Err_rTC2 (%)',...
    'Err_rTC (%)','Err_hSO (%)','Err_S (%)','Err_lambdaTE (%)','Err_alphaTE (%)','Err_lambdaM (%)','Err_alphaM (%)','Err_lambdaC (%)','Err_alphaC (%)',...
    'Err_lambdaSo (%)','Err_alphaSo (%)'};

ini=0; ini_2=0; h_SO=0;

freq1=struct1(:,1)';
Zreal_meas1=struct1(:,2)';
Zimag_meas1=struct1(:,3)';
Zmod_meas1=sqrt(struct1(:,2).^2+struct1(:,3).^2)';
Zphase_meas1=atan2(struct1(:,3),struct1(:,2))';

plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
grid on; axis equal; legend('Expermental data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')

struct2=struct1;
disp('Follow the instructions below to select the impedance data range to fit/simulate:'); disp(' ')
disp('1.- Magnify the plot to clearly see the initial point of the data range to be fitted/simulated.')
disp('2.- Press enter.')
disp('3.- Move the mouse and click so the data to be fitted is included in the top right quadrant.')
disp('4.- Magnify again the plot to clearly see now the final point of the data range to be fitted/simulated.')
disp('5.- Press enter.')
disp('6.- Move the mouse and click so the data to be fitted is included in the bottom left quadrant (you can click out of the white zone of the plot if needed).')
stop1=input('\nPress enter to continue...');
pts(1,:)=ginput(1);
stop1=input('\nPress enter to continue...');
pts(2,:)=ginput(1);
fileli=0;
for iii=1:length(freq1)
    if struct1(iii,2)<min(pts(1,1),pts(2,1))
        struct2(iii-fileli,:)=[]; fileli=fileli+1;
    elseif struct1(iii,2)>max(pts(1,1),pts(2,1))
        struct2(iii-fileli,:)=[]; fileli=fileli+1;
    elseif -struct1(iii,3)<min(pts(1,2),pts(2,2))
        struct2(iii-fileli,:)=[]; fileli=fileli+1;
    elseif -struct1(iii,3)>max(pts(1,2),pts(2,2))
        struct2(iii-fileli,:)=[]; fileli=fileli+1;
    end
end
freq2=struct2(:,1)'; Zreal_meas2=struct2(:,2)'; Zimag_meas2=struct2(:,3)';

hold off; plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
hold on; plot(Zreal_meas2,-Zimag_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
grid on; axis equal; legend('Expermental data','Selected data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')

while 1
    clc
    choice1=questdlg('Would you like to change the selected frequency range?','Range selection','Initial point','Final point','No','No');
    switch choice1
        case 'Initial point'
            disp('Follow the instructions below to select the impedance data range to fit/simulate:'); disp(' ')
            disp('1.- Magnify the plot to clearly see the initial point of the data range to be fitted/simulated.')
            disp('2.- Press enter.')
            disp('3.- Move the mouse and click so the data to be fitted is included in the top right quadrant.')
            stop1=input('\nPress enter to continue...');
            pts(1,:)=ginput(1);
        case 'Final point'
            disp('Follow the instructions below to select the impedance data range to fit/simulate:'); disp(' ')
            disp('1.- Magnify the plot to clearly see the final point of the data range to be fitted/simulated.')
            disp('2.- Press enter.')
            disp('3.- Move the mouse and click so the data to be fitted is included in the bottom left quadrant (you can click out of the white zone of the plot if needed).')
            stop1=input('\nPress enter to continue...');
            pts(2,:)=ginput(1);
        case 'No'
            break;
    end
    struct2=struct1;
    fileli=0;
    for iii=1:length(freq1)
        if struct1(iii,2)<min(pts(1,1),pts(2,1))
            struct2(iii-fileli,:)=[]; fileli=fileli+1;
        elseif struct1(iii,2)>max(pts(1,1),pts(2,1))
            struct2(iii-fileli,:)=[]; fileli=fileli+1;
        elseif -struct1(iii,3)<min(pts(1,2),pts(2,2))
            struct2(iii-fileli,:)=[]; fileli=fileli+1;
        elseif -struct1(iii,3)>max(pts(1,2),pts(2,2))
            struct2(iii-fileli,:)=[]; fileli=fileli+1;
        end
    end
    freq2=struct2(:,1)'; Zreal_meas2=struct2(:,2)'; Zimag_meas2=struct2(:,3)';

    hold off; plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
    hold on; plot(Zreal_meas2,-Zimag_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
    grid on; axis equal; legend('Expermental data','Selected data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')
end

w=2*pi*freq2;
Zmod_meas2=sqrt(struct2(:,2).^2+struct2(:,3).^2)';
Zphase_meas2=atan2(struct2(:,3),struct2(:,2))';

opts.Interpreter = 'tex'; dims=[1 100];

ff=figure('Resize','off');
left_sep=20; pos_last=50; pos_step=25;
long_sep=1300; alt_sep=20;
uicontrol(ff,'Style','text','Position',[10 350 540 50],'String',strcat('Select the properties you want to fit and press "Continue".',...
' Then, non-selected properties will take a fixed value that you should provide next (an initial value should be also provided for the properties to fit).',...
' If you want to run a simulation leave everything deselected.'));
LL_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+11*pos_step long_sep alt_sep],'String','Inductance (L)');
Rohm_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+10*pos_step long_sep alt_sep],'String','Ohmic resistance (Rohm)');
rTC1_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+9*pos_step long_sep alt_sep],'String','Thermal contact resistivities between thermoelements and metallic strips (rTC1)');
rTC2_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+8*pos_step long_sep alt_sep],'String','Thermal contact resistivities between metallic strips and ceramics (rTC2)');
rTC_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+7*pos_step long_sep alt_sep],'String','Thermal contact resistivities between ceramics and heat source/sink (rTC)');
S_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+6*pos_step long_sep alt_sep],'String','Average Seebeck coefficient of the thermoelements (S)');
k_TE_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+5*pos_step long_sep alt_sep],'String','Thermal conductivity of the thermoelements (lambdaTE)');
alpha_TE_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+4*pos_step long_sep alt_sep],'String','Thermal diffusivity of the thermoelements (alphaTE)');
k_cu_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+3*pos_step long_sep alt_sep],'String','Thermal conductivity of the metallic strips (lambdaM)');
alpha_cu_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+2*pos_step long_sep alt_sep],'String','Thermal diffusivity of the metallic strips (alphaM)');
k_C_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+pos_step long_sep alt_sep],'String','Thermal conductivity of the ceramics (lambdaC)');
alpha_C_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last long_sep alt_sep],'String','Thermal diffusivity of the ceramics (alphaC)');
hh=uicontrol('Position',[445 15 100 30],'String','Continue','Callback','uiresume(ff)');
uiwait(ff);
fit_prop=[LL_aux.Value Rohm_aux.Value rTC1_aux.Value rTC2_aux.Value rTC_aux.Value 0 S_aux.Value k_TE_aux.Value alpha_TE_aux.Value k_cu_aux.Value alpha_cu_aux.Value k_C_aux.Value alpha_C_aux.Value 0 0];
close(ff);

prompt_0={'Inital temperature (Ti) in K:','Inductance (L) in H:','Ohmic resistance (Rohm) in \Omega:',...
    'Thermal contact resistivities between thermoelements and metallic strips (r_{TC1}) in m^{2}K/W:',...
    'Thermal contact resistivities between metallic strips and ceramics (r_{TC2}) in m^{2}K/W:',...
    'Thermal contact resistivities between ceramics and heat source/sink (r_{TC}) in m^{2}K/W:'};
dlgtitle_0='Estimated values';
definput_0={'293','4e-7','1.25','1.01e-5','0','1e-4'};
answer_0=inputdlg(prompt_0,dlgtitle_0,dims,definput_0,opts);
if isempty(answer_0)==1
    error('Cancelled');
end
prompt_1={'Number of couples:','Length of the thermoelements (L) in m:',...
    'Thickness of the metallic strips (L_{M}) in m:','Thickness of the ceramics (L_{C}) in m:',...
    'Average area of the thermoelements (A) in m^{2}:','Ratio area legs/strips (\eta_{M}):',...
    'Filling factor of the thermoelectric module (\eta):'};
dlgtitle_1='Geometry of the thermoelectric module';
definput_1={'127','1.06e-3','0.22e-3','0.74e-3','1.34e-3*1.34e-3','0.67','0.27'};
answer_1=inputdlg(prompt_1,dlgtitle_1,dims,definput_1,opts);
if isempty(answer_1)==1
    error('Cancelled');
end
prompt_2={'Average Seebeck coefficient of the thermoelements (S) in V/K:',...
    'Thermal conductivity of the thermoelements (\lambda_{TE}) in W/(mK):',...
    'Thermal diffusivity of the thermoelements (\alpha_{TE}) in m^{2}/s:',...
    'Thermal conductivity of the metallic strips (\lambda_{M}) in W/(mK):',...
    'Thermal diffusivity of the metallic strips (\alpha_{M}) in m^{2}/s:',...
    'Thermal conductivity of the ceramics (\lambda_{C}) in W/(mK):',...
    'Thermal diffusivity of the ceramics (\alpha_{C}) in m^{2}/s:'};
dlgtitle_2='Material properties';
definput_2={'205.8e-6','1.955','3.7e-7','400','1.1e-4','30.29','0.1e-4'};
answer_2=inputdlg(prompt_2,dlgtitle_2,dims,definput_2,opts);
if isempty(answer_2)==1
    error('Cancelled');
end

[Ti,LL,Rohm,RC1,RC2,h3,N,L_TE,L_cu,L_C,A_te, eta_M,eta_C,S,k_TE,alpha_TE,k_cu,alpha_cu,k_C,alpha_C]=ext_vals(answer_0,answer_1,answer_2);

while 1
    if ini>0
        hold off; plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
        hold on; plot(Zreal_meas2,-Zimag_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
        grid on; axis equal; legend('Expermental data','Selected data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')
        
        while 1
            clc
            choice1=questdlg('Would you like to change the selected frequency range?','Range selection','Initial point','Final point','No','No');
            switch choice1
                case 'Initial point'
                    disp('Follow the instructions below to select the impedance data range to fit/simulate:'); disp(' ')
                    disp('1.- Magnify the plot to clearly see the initial point of the data range to be fitted/simulated.')
                    disp('2.- Press enter.')
                    disp('3.- Move the mouse and click so the data to be fitted is included in the top right quadrant.')
                    stop1=input('\nPress enter to continue...');
                    pts(1,:)=ginput(1);
                case 'Final point'
                    disp('Follow the instructions below to select the impedance data range to fit/simulate:'); disp(' ')
                    disp('1.- Magnify the plot to clearly see the final point of the data range to be fitted/simulated.')
                    disp('2.- Press enter.')
                    disp('3.- Move the mouse and click so the data to be fitted is included in the bottom left quadrant (you can click out of the white zone of the plot if needed).')
                    stop1=input('\nPress enter to continue...');
                    pts(2,:)=ginput(1);
                case 'No'
                    break;
            end
            struct2=struct1;
            fileli=0;
            for iii=1:length(freq1)
                if struct1(iii,2)<min(pts(1,1),pts(2,1))
                    struct2(iii-fileli,:)=[]; fileli=fileli+1;
                elseif struct1(iii,2)>max(pts(1,1),pts(2,1))
                    struct2(iii-fileli,:)=[]; fileli=fileli+1;
                elseif -struct1(iii,3)<min(pts(1,2),pts(2,2))
                    struct2(iii-fileli,:)=[]; fileli=fileli+1;
                elseif -struct1(iii,3)>max(pts(1,2),pts(2,2))
                    struct2(iii-fileli,:)=[]; fileli=fileli+1;
                end
            end
            freq2=struct2(:,1)'; Zreal_meas2=struct2(:,2)'; Zimag_meas2=struct2(:,3)';

            hold off; plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
            hold on; plot(Zreal_meas2,-Zimag_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
            grid on; axis equal; legend('Expermental data','Selected data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')
        end

        w=2*pi*freq2;
        Zmod_meas2=sqrt(struct2(:,2).^2+struct2(:,3).^2)';
        Zphase_meas2=atan2(struct2(:,3),struct2(:,2))';

        ff=figure('Resize','off');
        uicontrol(ff,'Style','text','Position',[10 350 540 50],'String',strcat('Select the properties you want to fit and press "Continue".',...
        ' Then, non-selected properties will take a fixed value that you should provide next (an initial value should be also provided for the properties to fit).',...
        ' If you want to run a simulation leave everything deselected.'));
        LL_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+11*pos_step long_sep alt_sep],'String','Inductance (L)');
        Rohm_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+10*pos_step long_sep alt_sep],'String','Ohmic resistance (Rohm)');
        rTC1_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+9*pos_step long_sep alt_sep],'String','Thermal contact resistivities between thermoelements and metallic strips (rTC1)');
        rTC2_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+8*pos_step long_sep alt_sep],'String','Thermal contact resistivities between metallic strips and ceramics (rTC2)');
        rTC_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+7*pos_step long_sep alt_sep],'String','Thermal contact resistivities between ceramics and heat source/sink (rTC)');
        S_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+6*pos_step long_sep alt_sep],'String','Average Seebeck coefficient of the thermoelements (S)');
        k_TE_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+5*pos_step long_sep alt_sep],'String','Thermal conductivity of the thermoelements (lambdaTE)');
        alpha_TE_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+4*pos_step long_sep alt_sep],'String','Thermal diffusivity of the thermoelements (alphaTE)');
        k_cu_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+3*pos_step long_sep alt_sep],'String','Thermal conductivity of the metallic strips (lambdaM)');
        alpha_cu_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+2*pos_step long_sep alt_sep],'String','Thermal diffusivity of the metallic strips (alphaM)');
        k_C_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last+pos_step long_sep alt_sep],'String','Thermal conductivity of the ceramics (lambdaC)');
        alpha_C_aux=uicontrol(ff,'Style','checkbox','Position',[left_sep pos_last long_sep alt_sep],'String','Thermal diffusivity of the ceramics (alphaC)');
        hh=uicontrol('Position',[445 15 100 30],'String','Continue','Callback','uiresume(ff)');
        uiwait(ff);
        fit_prop=[LL_aux.Value Rohm_aux.Value rTC1_aux.Value rTC2_aux.Value rTC_aux.Value 0 S_aux.Value k_TE_aux.Value alpha_TE_aux.Value k_cu_aux.Value alpha_cu_aux.Value k_C_aux.Value alpha_C_aux.Value 0 0];
        close(ff);

        definput_0={num2str(Ti),num2str(LL),num2str(Rohm),num2str(RC1),num2str(RC2),num2str(1/h3)};
        answer_0=inputdlg(prompt_0,dlgtitle_0,dims,definput_0,opts);
        if isempty(answer_0)==1
            error('Cancelled');
        end
        definput_1={num2str(N),num2str(L_TE),num2str(L_cu),num2str(L_C),num2str(A_te),num2str(eta_M),num2str(eta_C)};
        answer_1=inputdlg(prompt_1,dlgtitle_1,dims,definput_1,opts);
        if isempty(answer_1)==1
            error('Cancelled');
        end
        definput_2={num2str(S),num2str(k_TE),num2str(alpha_TE),num2str(k_cu),num2str(alpha_cu),num2str(k_C),num2str(alpha_C)};
        answer_2=inputdlg(prompt_2,dlgtitle_2,dims,definput_2,opts);
        if isempty(answer_2)==1
            error('Cancelled');
        end
        
        [Ti,LL,Rohm,RC1,RC2,h3,N,L_TE,L_cu,L_C,A_te, eta_M,eta_C,S,k_TE,alpha_TE,k_cu,alpha_cu,k_C,alpha_C]=ext_vals(answer_0,answer_1,answer_2);
    end
    ini=ini+1;
    
    OPTIONS=optimset('Display','off','TolX',1e-50,'TolFun',1e-50,'MaxIter',500);
    refine=min(struct2(:,2))*0.9;
    Zmod_meas2_ref=sqrt((struct2(:,2)-refine).^2+struct2(:,3).^2)';
    
    if h3~=0
        choice0=questdlg('Is the temperature constant in your heat source during the impedance measurement?','Study case','Yes','No','Yes');
        rTC=1/h3; L_SO=1e30; k_SO=1e30; alpha_SO=1e30;
        switch choice0
            case 'No'
                prompt_3={'Length of the heat source (L_{So}) in m:','Thermal conductivity of the heat source (\lambda_{So}) in W/(mK):',...
                    'Thermal diffusivity of the heat source (\alpha_{So}) in m^{2}/s:'};
                dlgtitle_3='Heat source parameters';
                definput_3={'19.5e-3','400','1.1e-4'};
                answer_3=inputdlg(prompt_3,dlgtitle_3,dims,definput_3,opts);
                if isempty(answer_3)==1
                    error('Cancelled');
                end
                L_SO=str2num(answer_3{1}); k_SO=str2num(answer_3{2}); alpha_SO=str2num(answer_3{3});
                
                choice3=questdlg('Would you like to fit the thermal conductivity or the thermal diffusivity of the heat source?',...
                    'Additional fitted parameters','Yes','No','No');
                switch choice3
                    case 'Yes'
                        choice4=questdlg('Which parameter do you want to fit?',...
                            'Additional fitted parameters','Thermal conductivity','Thermal diffusivity','Both','Both');
                        switch choice4
                            case 'Thermal conductivity'
                                fit_prop(end-1)=1; fit_prop(end)=0;
                            case 'Thermal diffusivity'
                                fit_prop(end-1)=0; fit_prop(end)=1;
                            case 'Both'
                                fit_prop(end-1)=1; fit_prop(end)=1;
                        end
                end
        end
    else
        rTC=1e30; L_SO=1e-30; k_SO=1e-30; alpha_SO=1e-30;
    end
    
    guesspars=[LL Rohm RC1 RC2 rTC h_SO S k_TE alpha_TE k_cu alpha_cu k_C alpha_C k_SO alpha_SO];
    limlow=guesspars; limhigh=guesspars; en_ter=find(fit_prop==1); limlow(en_ter)=0; limhigh(en_ter)=2*Inf;
    
    pass.w=w; pass.N=N; pass.Ti=Ti; pass.L_TE=L_TE; pass.L_cu=L_cu; pass.L_C=L_C; pass.A_te=A_te; pass.eta_M=eta_M; pass.eta_C=eta_C; pass.L_SO=L_SO;
    
    Simplified_2=@(val) abs((abs(fitequation(val(1),val(2),val(3),val(4),val(5),val(6),val(7),val(8),val(9),val(10),val(11),val(12),val(13),val(14),val(15),pass))-Zmod_meas2)./Zmod_meas2)+...
        abs((abs((fitequation(val(1),val(2),val(3),val(4),val(5),val(6),val(7),val(8),val(9),val(10),val(11),val(12),val(13),val(14),val(15),pass))-refine)-Zmod_meas2_ref)./Zmod_meas2_ref);
    
    disp('ATENTION: The fitting fails if it needs more frequency points!'); disp(' ')
    [fittedpars,resnorm,residual,exitflag,output,lambda,jacobian]=lsqnonlin(@(parlist)Simplified_2(parlist),guesspars,limlow,limhigh,OPTIONS);
    
    jacobianet=jacobian(:,en_ter); Cov=resnorm*inv((jacobianet'*jacobianet))/(length(residual)-length(en_ter)); Std=sqrt(diag(Cov)); relativeErr=Std./fittedpars(en_ter)'*100;
    
    if isempty(en_ter)
        clc; disp('ATENTION: All parameters were fixed and only a simulation is shown!')
    else
        gg={'---','---','---','---','---','---','---','---','---','---','---','---','---','---','---'};
        clc; disp('RESULTS OF THE FITTING:'); disp(' ')
        val_err=0;
        if fit_prop(1)==1
            val_err=val_err+1; gg{1}=num2str(relativeErr(val_err));
            disp(['Inductance (L) = ' num2str(fittedpars(1)) ' H' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(2)==1
            val_err=val_err+1; gg{2}=num2str(relativeErr(val_err));
            disp(['Ohmic resistance (Rohm) = ' num2str(fittedpars(2)) ' Ohm' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(3)==1
            val_err=val_err+1; gg{3}=num2str(relativeErr(val_err));
            disp(['Thermal contact resistivities between TE legs and metallic strips (rTC1) = ' num2str(fittedpars(3)) ' m^2K/W' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(4)==1
            val_err=val_err+1; gg{4}=num2str(relativeErr(val_err));
            disp(['Thermal contact resistivities between metallic strips and ceramics (rTC2) = ' num2str(fittedpars(4)) ' m^2K/W' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(5)==1
            val_err=val_err+1; gg{5}=num2str(relativeErr(val_err));
            disp(['Thermal contact resistivities between ceramics and heat source/sink (rTC) = ' num2str(1/fittedpars(5)) ' m^2K/W' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(6)==1
            val_err=val_err+1; gg{6}=num2str(relativeErr(val_err));
            disp(['Convection coefficient at the top of the liquid (hSo) = ' num2str(fittedpars(6)) ' W/m^2K' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(7)==1
            val_err=val_err+1; gg{7}=num2str(relativeErr(val_err));
            disp(['Average Seebeck coefficient of the thermoelements (S) = ' num2str(fittedpars(7)) ' V/K' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(8)==1
            val_err=val_err+1; gg{8}=num2str(relativeErr(val_err));
            disp(['Thermal conductivity of the thermoelements (lambdaTE) = ' num2str(fittedpars(8)) ' W/(mK)' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(9)==1
            val_err=val_err+1; gg{9}=num2str(relativeErr(val_err));
            disp(['Thermal diffusivity of the thermoelements (alphaTE) = ' num2str(fittedpars(9)) ' m^2/s' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(10)==1
            val_err=val_err+1; gg{10}=num2str(relativeErr(val_err));
            disp(['Thermal conductivity of the metallic strips (lambdaM) = ' num2str(fittedpars(10)) ' W/(mK)' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(11)==1
            val_err=val_err+1; gg{11}=num2str(relativeErr(val_err));
            disp(['Thermal diffusivity of the metallic strips (alphaM) = ' num2str(fittedpars(11)) ' m^2/s' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(12)==1
            val_err=val_err+1; gg{12}=num2str(relativeErr(val_err));
            disp(['Thermal conductivity of the ceramics (lambdaC) = ' num2str(fittedpars(12)) ' W/(mK)' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(13)==1
            val_err=val_err+1; gg{13}=num2str(relativeErr(val_err));
            disp(['Thermal diffusivity of the ceramics (alphaC) = ' num2str(fittedpars(13)) ' m^2/s' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(14)==1
            val_err=val_err+1; gg{14}=num2str(relativeErr(val_err));
            disp(['Thermal conductivity of the heat source (lambdaSo) = ' num2str(fittedpars(14)) ' W/(mK)' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        if fit_prop(15)==1
            val_err=val_err+1; gg{15}=num2str(relativeErr(val_err));
            disp(['Thermal diffusivity of the heat source (alphaSo) = ' num2str(fittedpars(15)) ' m^2/s' '          Error = ' num2str(relativeErr(val_err)) ' %'])
        end
        disp(' ')
        disp('To reduce fitting errors, consider fixing some parameters and/or modify the selected frequency range.')
        disp('In case you saved the results, check them with the above values (the .xls file may confuse "," and ".").')
        disp('You can also contact us if you need help (teslab@uji.es)!')
    end
    
    Z1=fitequation(fittedpars(1),fittedpars(2),fittedpars(3),fittedpars(4),fittedpars(5),fittedpars(6),fittedpars(7),fittedpars(8),fittedpars(9),fittedpars(10),...
        fittedpars(11),fittedpars(12),fittedpars(13),fittedpars(14),fittedpars(15),pass);

    hold off; plot(Zreal_meas1,-Zimag_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
    hold on; plot(Zreal_meas2,-Zimag_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
    plot(real(Z1),-imag(Z1),'-','Color',[0 0.5 0],'Marker','.','MarkerEdgeColor',[0 0.5 0],'MarkerSize',10)
    grid on; axis equal; legend('Expermental data','Selected data'); xlabel('Real Z (\Omega)'); ylabel('-Imag Z (\Omega)')
    figure; subplot(2,1,1)
    semilogx(freq1,Zmod_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
    hold on; semilogx(freq2,Zmod_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
    semilogx(freq2,abs(Z1),'-','Color',[0 0.5 0],'Marker','.','MarkerEdgeColor',[0 0.5 0],'MarkerSize',10)
    grid on; legend('Experimental data','Selected data','Fitting'); xlabel('Frequency (Hz)'); ylabel('Magnitude (\Omega)')
    subplot(2,1,2)
    semilogx(freq1,Zphase_meas1,':k','Marker','.','MarkerEdgeColor','k','MarkerSize',10)
    hold on; semilogx(freq2,Zphase_meas2,'-r','Marker','.','MarkerEdgeColor','b','MarkerSize',10)
    semilogx(freq2,angle(Z1),'-','Color',[0 0.5 0],'Marker','.','MarkerEdgeColor',[0 0.5 0],'MarkerSize',10)
    grid on; legend('Experimental data','Selected data','Fitting'); xlabel('Frequency (Hz)'); ylabel('Phase (rad)')
    
    if max(fit_prop)>0
        choice2=questdlg('Would you like to save the results in a .xls file?',...
            'Save','Yes','No','No');
        switch choice2
            case 'Yes'
                ee={num2str(fittedpars(1)),num2str(fittedpars(2)),num2str(fittedpars(3)),num2str(fittedpars(4)),...
                    num2str(fittedpars(5)),num2str(fittedpars(6)),num2str(fittedpars(7)),...
                    num2str(fittedpars(8)),num2str(fittedpars(9)),num2str(fittedpars(10)),num2str(fittedpars(11)),...
                    num2str(fittedpars(12)),num2str(fittedpars(13)),num2str(fittedpars(14)),num2str(fittedpars(15)),...
                    gg{1},gg{2},gg{3},gg{4},gg{5},gg{6},gg{7},gg{8},gg{9},gg{10},gg{11},gg{12},gg{13},gg{14},gg{15}};
                dd=[dd;ee];
                ini_2=1;
        end
    end

    stop2=input('\nTo stop (and create the .xls file, if applicable) type "exit"... To continue just press enter...\n','s');
    if strcmp(stop2,'exit')||strcmp(stop2,'Exit')||strcmp(stop2,'EXIT')||strcmp(stop2,'"exit"')
        break;
    end
    close all
end

if ini_2==1
    real_name_file=name_file(1:length(name_file)-4); xlswrite(real_name_file,dd)
end

choice3=questdlg('Would you like to save the last fitted data in a .xls file?',...
    'Save','Yes','No','No');
switch choice3
    case 'Yes'
        writematrix([freq2',real(Z1)',imag(Z1)'],'Fitted_data.csv')
    case 'No'
end

function [Ti,LL,Rohm,RC1,RC2,h3,N,L_TE,L_cu,L_C,A_te, eta_M,eta_C,S,k_TE,alpha_TE,k_cu,alpha_cu,k_C,alpha_C]=ext_vals(answer_0,answer_1,answer_2)
Ti=str2num(answer_0{1}); LL=str2num(answer_0{2}); Rohm=str2num(answer_0{3}); RC1=str2num(answer_0{4}); RC2=str2num(answer_0{5}); h3=1/str2num(answer_0{6});
N=str2num(answer_1{1}); L_TE=str2num(answer_1{2}); L_cu=str2num(answer_1{3}); L_C=str2num(answer_1{4}); A_te=str2num(answer_1{5}); eta_M=str2num(answer_1{6}); eta_C=str2num(answer_1{7});
S=str2num(answer_2{1}); k_TE=str2num(answer_2{2}); alpha_TE=str2num(answer_2{3}); k_cu=str2num(answer_2{4}); alpha_cu=str2num(answer_2{5}); k_C=str2num(answer_2{6}); alpha_C=str2num(answer_2{7});
end

function Ztotal=fitequation(LL,Rohm,RC1,RC2,rTC,h_SO,Sbck,k_TE,alpha_TE,k_cu,alpha_cu,k_C,alpha_C,k_SO,alpha_SO,pass)
rTCSo=rTC; rTCSi=rTC;

w=pass.w; N=pass.N; Ti=pass.Ti; L_TE=pass.L_TE; L_cu=pass.L_cu; L_C=pass.L_C; A_te=pass.A_te; eta_M=pass.eta_M; eta_C=pass.eta_C; L_SO=pass.L_SO;

exp_plus=(1+exp(-2*((1i*w./(alpha_TE/(L_TE^2))).^0.5)))/2;
exp_minus=(1-exp(-2*((1i*w./(alpha_TE/(L_TE^2))).^0.5)))/2;
ZTOT5t_exp=k_TE*A_te/L_TE*(1i*w./(alpha_TE/(L_TE^2))).^0.5;
exp_simp=exp(-((1i*w./(alpha_TE/(L_TE^2))).^0.5));
Flux=Sbck*Ti;

x1=sqrt(A_te/eta_C)/2+sqrt(A_te)/2; y1=sqrt(A_te)/2; x2=sqrt(A_te/eta_C); y2=sqrt(A_te/eta_C)/2;
njj=(1:100)'; mjj=(1:100); mat_njj=repmat(njj,1,length(mjj)); mat_mjj=repmat(mjj,length(njj),1);

for each_w=1:length(w)
    w_aux_nn=repmat(w(each_w),length(njj),1);
    w_aux_mm=repmat(w(each_w),1,length(mjj));
    w_aux=repmat(w(each_w),length(njj),length(mjj));
    gamma_nn=sqrt((njj*pi/x2).^2+(1j*w_aux_nn/alpha_C));
    gamma_mm=sqrt((mjj*pi/y2).^2+(1j*w_aux_mm/alpha_C));
    gamma_sq=sqrt((mat_njj*pi/x2).^2+(mat_mjj*pi/y2).^2+(1j*w_aux/alpha_C));
    num_nn=x2*y1*sin(njj*pi*x1/x2).^2;
    num_mm=y2*x1*sin(mjj*pi*y1/y2).^2;
    num_sq=x2*y2*sin(mat_njj*pi*x1/x2).^2.*sin(mat_mjj*pi*y1/y2).^2;
    
    par_RSo_nn=(k_C.*gamma_nn.*tanh(gamma_nn.*L_C)+(1/rTCSo))./(k_C.*gamma_nn+(1/rTCSo).*tanh(gamma_nn.*L_C));
    par_RSo_mm=(k_C.*gamma_mm.*tanh(gamma_mm.*L_C)+(1/rTCSo))./(k_C.*gamma_mm+(1/rTCSo).*tanh(gamma_mm.*L_C));
    par_RSo=(k_C.*gamma_sq.*tanh(gamma_sq.*L_C)+(1/rTCSo))./(k_C.*gamma_sq+(1/rTCSo).*tanh(gamma_sq.*L_C));
    den_sqSo_nn=x1*y2*pi^2*njj.^2.*gamma_nn.*par_RSo_nn;
    den_sqSo_mm=y1*x2*pi^2*mjj.^2.*gamma_mm.*par_RSo_mm;
    den_sqSo=x1*y1*pi^4*mat_njj.^2.*mat_mjj.^2.*gamma_sq.*par_RSo;
    ZSC4NS2TMSo(each_w)=4*(eta_M/(k_C*A_te))*sum(sum(num_sq./den_sqSo))+2*(eta_M/(k_C*A_te))*sum(num_nn./den_sqSo_nn)...
        +2*(eta_M/(k_C*A_te))*sum(num_mm./den_sqSo_mm);
    
    par_RSi_nn=(k_C.*gamma_nn.*tanh(gamma_nn.*L_C)+(1/rTCSi))./(k_C.*gamma_nn+(1/rTCSi).*tanh(gamma_nn.*L_C));
    par_RSi_mm=(k_C.*gamma_mm.*tanh(gamma_mm.*L_C)+(1/rTCSi))./(k_C.*gamma_mm+(1/rTCSi).*tanh(gamma_mm.*L_C));
    par_RSi=(k_C.*gamma_sq.*tanh(gamma_sq.*L_C)+(1/rTCSi))./(k_C.*gamma_sq+(1/rTCSi).*tanh(gamma_sq.*L_C));
    den_sqSi_nn=x1*y2*pi^2*njj.^2.*gamma_nn.*par_RSi_nn;
    den_sqSi_mm=y1*x2*pi^2*mjj.^2.*gamma_mm.*par_RSi_mm;
    den_sqSi=x1*y1*pi^4*mat_njj.^2.*mat_mjj.^2.*gamma_sq.*par_RSi;
    ZSC4NS2TMSi(each_w)=4*(eta_M/(k_C*A_te))*sum(sum(num_sq./den_sqSi))+2*(eta_M/(k_C*A_te))*sum(num_nn./den_sqSi_nn)...
        +2*(eta_M/(k_C*A_te))*sum(num_mm./den_sqSi_mm);
end

RTC14NS2TM=RC1/A_te;
RTC24NS2TM=RC2*eta_M/A_te;
ZWaM4NS2TM=(L_cu*eta_M/(k_cu*A_te)).*(1i*w./(alpha_cu/(L_cu^2))).^-0.5.*coth((1i*w./(alpha_cu/(L_cu^2))).^0.5);
ZWCTM4NS2TM=(L_cu*eta_M/(k_cu*A_te)).*(1i*w./(alpha_cu/(L_cu^2))).^-0.5.*tanh((1i*w./(alpha_cu/(L_cu^2))).^0.5);
ZWCTC4NS2TM=(L_C*eta_C/(k_C*A_te)).*(1i*w./(alpha_C/(L_C^2))).^-0.5.*tanh((1i*w./(alpha_C/(L_C^2))).^0.5);
ZWa4NS2TM=(L_C*eta_C/(k_C*A_te)).*(1i*w./(alpha_C/(L_C^2))).^-0.5.*coth((1i*w./(alpha_C/(L_C^2))).^0.5);
RhSO4NS2TM=eta_C/(h_SO*A_te);
ZWaSo4NS2TM=(L_SO*eta_C/(k_SO*A_te)).*(1i*w./(alpha_SO/(L_SO^2))).^-0.5.*coth((1i*w./(alpha_SO/(L_SO^2))).^0.5);
ZWCTSO4NS2TM=(L_SO*eta_C/(k_SO*A_te)).*(1i*w./(alpha_SO/(L_SO^2))).^-0.5.*tanh((1i*w./(alpha_SO/(L_SO^2))).^0.5);

RTC4NS2TMSo=eta_C*rTCSo/A_te;
ZChSO=(L_SO*eta_C/(k_SO*A_te))^2/RhSO4NS2TM.*(1i*w./(alpha_SO/(L_SO^2))).^-1;
ZWaSoMNoConv=(ZWaSo4NS2TM.^-1+RhSO4NS2TM.^-1).^-1+(ZWCTSO4NS2TM.^-1+ZChSO.^-1).^-1;
ZTOT3=RTC24NS2TM+ZSC4NS2TMSo+(ZWa4NS2TM.^-1+(RTC4NS2TMSo+ZWaSoMNoConv).^-1).^-1+(ZWCTC4NS2TM.^-1+...
    ((L_C*eta_C/(k_C*A_te)).^2./(RTC4NS2TMSo+ZWaSoMNoConv).*(1i*w./(alpha_C/(L_C^2))).^-1).^-1).^-1;
ZSo=RTC14NS2TM+((ZWaM4NS2TM.^-1+ZTOT3.^-1).^-1+(ZWCTM4NS2TM.^-1+...
    ((L_cu*eta_M/(k_cu*A_te)).^2./ZTOT3.*(1i*w./(alpha_cu/(L_cu^2))).^-1).^-1).^-1);

RTC4NS2TMSi=eta_C*rTCSi/A_te;
ZCTC4NS2TM=(L_C*eta_C/(k_C*A_te))^2/RTC4NS2TMSi.*(1i*w./(alpha_C/(L_C^2))).^-1;
ZTOT5=RTC24NS2TM+ZSC4NS2TMSi+(ZWa4NS2TM.^-1+RTC4NS2TMSi.^-1).^-1+(ZWCTC4NS2TM.^-1+ZCTC4NS2TM.^-1).^-1;
ZTOT6_1=(L_cu*eta_M/(k_cu*A_te)).^2./ZTOT5.*(1i*w./(alpha_cu/(L_cu^2))).^-1;
ZSi=RTC14NS2TM+(ZWaM4NS2TM.^-1+ZTOT5.^-1).^-1+(ZWCTM4NS2TM.^-1+ZTOT6_1.^-1).^-1;

Temp0_num=Flux.*exp_plus+Flux.*exp_minus./ZTOT5t_exp./ZSi-Flux.*exp_simp;
Temp0_den=ZTOT5t_exp.*exp_minus+exp_plus./ZSi+exp_plus./ZSo+exp_minus./ZTOT5t_exp./ZSi./ZSo;
Temp0=Temp0_num./Temp0_den;
TempL_num=Flux*exp_simp-Temp0.*exp_simp./ZSo-exp_plus*Flux;
TempL_den=ZTOT5t_exp.*exp_minus+exp_plus./ZSi;
TempL=TempL_num./TempL_den;

Ztotal=1i*w.*LL+Rohm+2*N*Sbck*(Temp0-TempL);
end
